###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""The ClientTools module provides additional functions for use by users of the
DIRAC client in the LHCb environment."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import tempfile

from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.Utilities.List import breakListIntoChunks

from LHCbDIRAC.Core.Utilities.RunApplication import RunApplication

__RCSID__ = "$Id$"


def _errorReport(error, message=None):
  """Internal function to return errors and exit with an S_ERROR()"""
  if not message:
    message = error

  gLogger.warn(error)
  return S_ERROR(message)


def mergeRootFiles(outputFile, inputFiles):
  """Merge several ROOT files.

  Args:
      outputFile (str): output file name
      inputFiles (list): list of input files
  """
  if not isinstance(inputFiles, list):
    return _errorReport("please provide a list of input files")

  # Performs the merging
  chunkSize = 20

  if len(inputFiles) > chunkSize:  # 2-steps merge
    lists = breakListIntoChunks(inputFiles, chunkSize)
    tempFiles = []
    for filelist in lists:
      fd, fn = tempfile.mkstemp()
      os.close(fd)
      tempFiles.append(fn)
      res = _mergeRootFiles(fn, filelist)
      if not res['OK']:
        return _errorReport(res['Message'], "Failed to perform ROOT merger")
    res = _mergeRootFiles(outputFile, tempFiles)
    if not res['OK']:
      return _errorReport(res['Message'], "Failed to perform final ROOT merger")
    # Cleaning up temporary files
    for filename in tempFiles:
      if os.path.exists(filename):
        os.remove(filename)

  else:  # merge in one go
    res = _mergeRootFiles(outputFile, inputFiles)
    if not res['OK']:
      return _errorReport(res['Message'], "Failed to perform ROOT merger")

  return S_OK(outputFile)

#############################################################################


def _mergeRootFiles(outputFile, inputFiles):
  """Merge ROOT files.

  Args:
      outputFile (str): output file name
  """
  cmd = "hadd -f %s " % outputFile + ' '.join(inputFiles)

  ra = RunApplication()
  ra.applicationName = 'ROOT'
  ra.command = cmd

  try:
    ra.run()
    return S_OK()
  except RuntimeError:
    return _errorReport("Failed to merge root files")



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

"""Base Entity System Manager."""


import os
from DIRAC import gLogger, S_OK, S_ERROR


__RCSID__ = "$Id$"


class BaseESManager(object):
  """Base Entity manager class."""

  #############################################################################
  def __init__(self):
    """Initialize the class members."""
    self.__fileSeparator = '/'

  #############################################################################
  def getPathSeparator(self):
    """The path separator used."""
    return self.__fileSeparator

  #############################################################################
  def list(self, path="/", selectionDict=None, sortDict=None, startItem=0, maxitems=0):
    """list the path."""
    selectionDict = selectionDict if selectionDict is not None else {}
    sortDict = sortDict if sortDict is not None else {}
    gLogger.error('This method is not implemented!' + (str(self.__class__)))
    gLogger.error(str(path))
    gLogger.error(str(selectionDict))
    gLogger.error(str(sortDict))
    gLogger.error(str(startItem))
    gLogger.error(str(maxitems))
    return S_ERROR("Not Implemented!")

  #############################################################################
  @staticmethod
  def getAbsolutePath(path):
    """absolute path."""
    # get current working directory if empty
    if path == "" or path is None:
      path = "."
      # convert it into absolute path
    try:
      path = os.path.abspath(path)
      return S_OK(path)
    except IOError as ex:
      return S_ERROR("getAbsalutePath: " + str(ex))

  #############################################################################
  def mergePaths(self, path1, path2):
    """merge two path."""
    gLogger.debug("mergePaths(path1, path2) with input " + str(path1) + ", " + str(path2))
    path = self.getAbsolutePath(os.path.join(path1, path2))
    return path

  #############################################################################
  def get(self, path=""):
    """the path element."""
    gLogger.warn('not implemented' + path + str(self.__class__))
    return S_ERROR("Not implemented!")



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""stores the simulation condition."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import gLogger

__RCSID__ = "$Id$"


class SimulationConditions:
  """SimulationConditions class."""
  #############################################################################

  def __init__(self):
    """initialize the class member."""
    self.parameters_ = {}

  #############################################################################
  def addParam(self, name, value):
    """adds a parameter into the dictionary."""
    self.parameters_[name] = value

  #############################################################################
  def getParams(self):
    """returns the parameters."""
    return self.parameters_

  #############################################################################
  def writeToXML(self):
    """creates the xml string."""
    gLogger.info("Write Simulation conditions to XML!!")
    result = '<SimulationCondition>\n'
    for name, value in self.getParams().items():
      result += '    <Parameter Name="' + name + '"   Value="' + value + '"/>\n'
    result += '</SimulationCondition>\n'

    return result



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Retrieve from Bookkeeping information for a given run."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger, S_OK
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script

  Script.registerSwitch('', 'Production=', '   <prodID>, get the run list from a production')
  Script.registerSwitch('', 'Active', '   only get Active runs')
  Script.registerSwitch('', 'Information=', '   <item> returns only the relevant information item')
  Script.registerSwitch(
      '',
      'ByValue',
      '   if set, the information is a list of runs for each value of the information item')
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] ... Run' % Script.scriptName,
      'Arguments:',
      '  Run:      Run Number']))
  Script.parseCommandLine(ignoreErrors=True)
  runRanges = []
  for arg in Script.getPositionalArgs():
    runRanges += arg.split(',')

  runSet = set()
  for run in runRanges:
    try:
      if ':' in arg:
        run1, run2 = run.split(':')
        runSet.update(range(int(run1), int(run2) + 1))
      else:
        runSet.add(int(run))
    except (ValueError, IndexError) as e:
      gLogger.exception("Invalid run number", arg, lException=e)
      DIRAC.exit(1)

  production = None
  item = None
  byValue = False
  active = False
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'Production':
      try:
        production = [int(prod) for prod in switch[1].split(',')]
      except ValueError as e:
        gLogger.exception('Bad production ID', lException=e)
        DIRAC.exit(1)
    elif switch[0] == 'Information':
      item = switch[1]
    elif switch[0] == 'ByValue':
      byValue = True
    elif switch[0] == 'Active':
      active = True

  from LHCbDIRAC.DataManagementSystem.Client.DMScript import printDMResult, ProgressBar
  from LHCbDIRAC.BookkeepingSystem.Client.BookkeepingClient import BookkeepingClient
  bk = BookkeepingClient()

  if production:
    from LHCbDIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
    trClient = TransformationClient()
    condDict = {'TransformationID': production}
    if active:
      condDict['Status'] = 'Active'
    res = trClient.getTransformationRuns(condDict)
    if res['OK']:
      runSet.update(run['RunNumber'] for run in res['Value'])
    else:
      gLogger.fatal("Error getting production runs", res['Message'])
      DIRAC.exit(2)
    gLogger.notice("Found %d runs" % len(runSet))

  # Use this call to get information but also the actual list of existing runs
  res = bk.getRunStatus(list(runSet))
  if not res['OK']:
    gLogger.fatal("Error getting the run info", res['Message'])
    DIRAC.exit(2)
  runStatus = res['Value']['Successful']

  sep = ''
  if item:
    result = {"Successful": {}, "Failed": {}}
    success = result['Successful']
    failed = result['Failed']
    if item == 'Finished':
      # We can get it in a single call
      for run in runStatus:
        finished = {'N': 'No', 'Y': 'Yes'}.get(runStatus.get(run, {}).get('Finished'), 'Unknown')
        if byValue:
          success.setdefault(finished, []).append(run)
        else:
          success[run] = finished
    else:
      progressBar = ProgressBar(len(runStatus), title="Getting %s for %d runs" % (item, len(runStatus)), step=10)

  if item != 'Finished':
    for run in sorted(runStatus):
      if item:
        progressBar.loop()
      finished = {'N': 'No', 'Y': 'Yes'}.get(runStatus[run].get('Finished'), 'Unknown')

      res = bk.getRunInformations(run)
      if res['OK']:
        info = res['Value']
        if item:
          if item not in info:
            gLogger.error("Item not found", "\n Valid items: %s" % str(sorted(info)))
            DIRAC.exit(3)
          itemValue = info.get(item, "Unknown")
          if byValue:
            success.setdefault(itemValue, []).append(run)
          else:
            result['Successful'][run] = itemValue
          continue
        runstart = info.get('RunStart', 'Unknown')
        runend = info.get('RunEnd', 'Unknown')
        configname = info.get('Configuration Name', 'Unknown')
        configversion = info.get('Configuration Version', 'Unknown')
        fillnb = info.get('FillNumber', 'Unknown')
        datataking = info.get('DataTakingDescription', 'Unknown')
        processing = info.get('ProcessingPass', 'Unknown')
        stream = info.get('Stream', 'Unknown')
        fullstat = info.get('FullStat', 'Unknown')
        nbofe = info.get('Number of events', 'Unknown')
        nboff = info.get('Number of file', 'Unknown')
        fsize = info.get('File size', 'Unknown')
        totalLuminosity = info.get("TotalLuminosity", 'Unknown')
        tck = info.get('Tck', 'Unknown')

        if sep:
          print(sep)
        print("Run  Informations for run %d: " % run)
        print("Run Start:".ljust(30), str(runstart))
        print("Run End:".ljust(30), str(runend))
        print("Total luminosity:".ljust(30), str(totalLuminosity))
        print("  Configuration Name:".ljust(30), configname)
        print("  Configuration Version:".ljust(30), configversion)
        print("  FillNumber:".ljust(30), fillnb)
        print("  Finished:".ljust(30), finished)
        print("  Data taking description:".ljust(30), datataking)
        print("  Processing pass:".ljust(30), processing)
        print("  TCK:".ljust(30), tck)
        print("  Stream:".ljust(30), stream)
        just = len(str(fsize)) + 3
        print("  FullStat:".ljust(30), str(fullstat).ljust(just), " Total: ".ljust(10) + str(sum(fullstat)))
        print("  Number of events:".ljust(30), str(nbofe).ljust(just), " Total:".ljust(10) + str(sum(nbofe)))
        print("  Number of files:".ljust(30), str(nboff).ljust(just), " Total: ".ljust(10) + str(sum(nboff)))
        print("  File size:".ljust(30), str(fsize).ljust(just), " Total: ".ljust(10) + str(sum(fsize)))
        sep = 20 * '='
      elif item:
        failed[run] = res['Message']
    if item:
      progressBar.endLoop()

  if item:
    if not failed:
      del failed
    if byValue:
      for val in success:
        success[val] = ('(%d runs) - ' % len(success[val])) + (','.join(sorted(str(run) for run in success[val])))
    printDMResult(S_OK(result), empty='None', script='dirac-bookkeeping-run-information')


if __name__ == "__main__":
  main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Uses the DM script switches, and, unless a list of LFNs is provided

1. If --Directory is used: get files in FC directories, check if they are in BK and if the replica flag is set
2. If --Production is used get files in the FC directories used, and proceed as with --Directory

If --FixFC is set, remove from SE and FC
If --FixBK is set and no replica flag, set replica flag in the BK
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  # Script initialization
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, Script
  import DIRAC

  Script.setUsageMessage('\n'.join([__doc__,
                                    'Usage:',
                                    '  %s [option|cfgfile] [values]' % Script.scriptName, ]))
  dmScript = DMScript()
  dmScript.registerNamespaceSwitches()  # Directory
  dmScript.registerFileSwitches()  # File, LFNs
  dmScript.registerBKSwitches()
  Script.registerSwitch('', 'FixBK', '   Take action to fix the BK')
  Script.registerSwitch('', 'FixFC', '   Take action to fix the FC')
  Script.registerSwitch('', 'NoFC2SE', "   Don't execute an FC2SE check")
  Script.registerSwitch('', 'AffectedRuns', '   List the runs affected by the encountered problem')
  Script.parseCommandLine(ignoreErrors=True)

  from DIRAC import gLogger
  fixBK = False
  fixFC = False
  checkFC2SE = True
  listAffectedRuns = False
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'FixFC':
      fixFC = True
    if switch[0] == 'FixBK':
      fixBK = True
    elif switch[0] == 'AffectedRuns':
      listAffectedRuns = True
    elif switch[0] == 'NoFC2SE':
      checkFC2SE = False

  if fixFC and fixBK:
    gLogger.notice("Can't fix both FC and BK, please choose")
    DIRAC.exit(0)
  # imports
  from LHCbDIRAC.DataManagementSystem.Client.ConsistencyChecks import ConsistencyChecks
  cc = ConsistencyChecks()
  cc.directories = dmScript.getOption('Directory', [])
  cc.lfns = dmScript.getOption('LFNs', []) + [lfn for arg in Script.getPositionalArgs() for lfn in arg.split(',')]
  productions = dmScript.getOption('Productions', [])
  runs = dmScript.getOption('Runs', [])

  from LHCbDIRAC.DataManagementSystem.Client.CheckExecutors import doCheckFC2BK
  if productions and not runs:
    fileType = dmScript.getOption('FileType', [])
    if fileType:
      cc.fileType = fileType
    for prod in productions:
      cc.prod = prod
      gLogger.always("Processing production %d" % cc.prod)
      doCheckFC2BK(cc, fixFC, fixBK, listAffectedRuns)
      gLogger.always("Processed production %d" % cc.prod)
  else:
    bkQuery = dmScript.getBKQuery(visible='All')
    if bkQuery:
      bkQuery.setOption('ReplicaFlag', 'All')
      cc.bkQuery = bkQuery
    doCheckFC2BK(cc, fixFC=fixFC, fixBK=fixBK, listAffectedRuns=listAffectedRuns, checkFC2SE=checkFC2SE)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""stores the jobs and its parameters."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"


class Job:
  """Job class."""

  #############################################################################
  def __init__(self):
    """initialize the class members."""
    self.jobConfiguration_ = None
    self.jobOptions_ = []
    self.jobParameters_ = []
    self.jobInputFiles_ = []
    self.jobOutputfiles_ = []
    self.simulationCond_ = None
    self.dataTaking_ = None
    self.jobId_ = -1
    self.name_ = ""
    self.file_name_ = ""

  #############################################################################
  def setJobConfiguration(self, configuration):
    """sets the job configuration."""
    self.jobConfiguration_ = configuration

  #############################################################################
  def getJobConfiguration(self):
    """returns the job configuration."""
    return self.jobConfiguration_

  #############################################################################
  def addJobOptions(self, jobOption):
    """adds the job options."""
    self.jobOptions_ += [jobOption]

  #############################################################################
  def getJobOptions(self):
    """returns the job options."""
    return self.jobOptions_

  #############################################################################
  def addJobParams(self, jobParams):
    """sets the job parameters."""
    self.jobParameters_ += [jobParams]

  #############################################################################
  def removeJobParam(self, paramName):
    """remove a job parameter."""
    self.jobParameters_.remove(paramName)

  #############################################################################
  def getJobParams(self):
    """returns the job parameters."""
    return self.jobParameters_

  #############################################################################
  def exists(self, jobParam):
    """checks a given job parameter."""
    return any(i.getName() == jobParam for i in self.jobParameters_)

  #############################################################################
  def getParam(self, jobParam):
    """returns a job parameter."""
    param = None
    for i in self.jobParameters_:
      if i.getName() == jobParam:
        param = i
    return param

  #############################################################################
  def removeParam(self, jobParam):
    """removes a job parameter."""
    for i in self.jobParameters_:
      if i.getName() == jobParam:
        self.jobParameters_.remove(i)

  #############################################################################
  def addJobInputFiles(self, files):
    """adds the input file to a job."""
    self.jobInputFiles_ += [files]

  #############################################################################
  def getJobInputFiles(self):
    """returns the input files."""
    return self.jobInputFiles_

  #############################################################################
  def addJobOutputFiles(self, files):
    """adds the output files to a job."""
    self.jobOutputfiles_ += [files]

  #############################################################################
  def getJobOutputFiles(self):
    """returns the output files."""
    return self.jobOutputfiles_

  #############################################################################
  def getOutputFileParam(self, paramName):
    """returns the parameters of a output file."""
    for i in self.jobOutputfiles_:
      param = i.getParam(paramName)
      if param is not None:
        return param
    return None

  #############################################################################
  def addSimulationCond(self, cond):
    """sets the simulation condition of a job."""
    self.simulationCond_ = cond

  #############################################################################
  def addDataTakingCond(self, cond):
    """sets the data taking conditions of a job."""
    self.dataTaking_ = cond

  #############################################################################
  def getDataTakingCond(self):
    """returns the data taking conditions."""
    return self.dataTaking_
  #############################################################################

  def getSimulationCond(self):
    """returns the simulation conditions."""
    return self.simulationCond_

  #############################################################################
  def setFileName(self, name):
    """sets the file name."""
    self.file_name_ = name

  #############################################################################
  def getFileName(self):
    """returns the file name."""
    return self.file_name_

  #############################################################################
  def setJobId(self, jobid):
    """sets the job identifier."""
    self.jobId_ = jobid

  #############################################################################
  def getJobId(self):
    """returns the job identifier."""
    return self.jobId_

  #############################################################################
  def setJobName(self, name):
    """sets the job name."""
    self.name_ = name

  #############################################################################
  def getJobName(self):
    """returns the job name."""
    return self.name_

  #############################################################################
  def __repr__(self):
    """formats the output of the print command."""
    result = ["JOB: \n"]
    result += [str(self.jobConfiguration_) + " "]
    for option in self.jobOptions_:
      result += [str(option)]
    result += ['\n']
    for param in self.jobParameters_:
      result += [str(param)]
    result += ['\n']
    for jobinput in self.jobInputFiles_:
      result += [str(jobinput)]
    for output in self.jobOutputfiles_:
      result += [str(output)]
    result += ['\n']
    return "".join(result)

  #############################################################################
  def writeToXML(self):
    """writes an XML string."""
    string = []
    string += ['<?xml version="1.0" encoding="ISO-8859-1"?>\n']
    string += ['<!DOCTYPE Job SYSTEM "book.dtd">\n']

    string += [str(self.getJobConfiguration().writeToXML())]
    for param in self.jobParameters_:
      string += [str(param.writeToXML())]

    for inputFile in self.jobInputFiles_:
      string += [str(inputFile.writeToXML())]

    for output in self.jobOutputfiles_:
      string += [str(output.writeToXML())]

    sim = self.getSimulationCond()
    if sim is not None:
      string += [str(sim.writeToXML())]

    daq = self.getDataTakingCond()
    if daq is not None:
      string += [str(daq.writeToXML())]

    string += ['</Job>']

    return "".join(string)
#############################################################################



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""
:mod: DataUsageHandler

.. module: DataUsageHandler

:synopsis: Implementation of the Data Usage service in the DISET framework.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
import six
# # from DIRAC
from DIRAC import S_OK
from DIRAC.Core.DISET.RequestHandler import RequestHandler
# # from LHCbDIRAC
from LHCbDIRAC.DataManagementSystem.DB.StorageUsageDB import StorageUsageDB

# # RCSID
__RCSID__ = "$Id$"

# global instance of the StorageUsageDB class
gStorageUsageDB = False


def initializeDataUsageHandler(_serviceInfo):
  """service initalisation."""
  global gStorageUsageDB
  gStorageUsageDB = StorageUsageDB()
  return S_OK()


class DataUsageHandler(RequestHandler):
  """
  .. class:: DataUsageHandler
  """
  types_sendDataUsageReport = [six.string_types, dict]

  @staticmethod
  def export_sendDataUsageReport(site, directoryDict, status='New'):
    """export of sendDataUsageReport."""
    return gStorageUsageDB.sendDataUsageReport(site, directoryDict, status)

  types_getDataUsageSummary = [six.string_types, six.string_types, six.string_types]

  @staticmethod
  def export_getDataUsageSummary(startTime, endTime, status):
    """export of getDataUsageSummary."""
    return gStorageUsageDB.getDataUsageSummary(startTime, endTime, status)

  types_getDataUsageForDirectory = [six.string_types]

  @staticmethod
  def export_getDataUsageForDirectory(path):
    """export of getDataUsageForDirectory."""
    return gStorageUsageDB.getDataUsageForDirectory(path)

  types_sendDataUsageReport_2 = [(dict)]

  @staticmethod
  def export_sendDataUsageReport_2(directoryDict):
    """export of sendDataUsageReport (new version)"""
    return gStorageUsageDB.sendDataUsageReport_2(directoryDict)

  types_updatePopEntryStatus = [list, six.string_types]

  @staticmethod
  def export_updatePopEntryStatus(idList, newStatus):
    """export of updatePopEntryStatus."""
    return gStorageUsageDB.updatePopEntryStatus(idList, newStatus)

  types_insertToDirMetadata = [dict]

  @staticmethod
  def export_insertToDirMetadata(directoryDict):
    """export of insertToDirMetadata."""
    return gStorageUsageDB.insertToDirMetadata(directoryDict)

  types_getDirMetadata = [list]

  @staticmethod
  def export_getDirMetadata(directoryList):
    """export of getDirMetadata."""
    return gStorageUsageDB.getDirMetadata(directoryList)



#! /usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Get statistical information on a dataset."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, Script

  dmScript = DMScript()
  dmScript.registerBKSwitches()
  dmScript.registerFileSwitches()

  Script.registerSwitch('', 'TriggerRate', '   For RAW files, returns the trigger rate')
  Script.registerSwitch('', 'ListRuns', '   Give a list of runs (to be used with --Trigger)')
  Script.registerSwitch('', 'ListFills', '   Give a list of fills (to be used with --Trigger)')
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile]' % Script.scriptName, ]))

  Script.parseCommandLine(ignoreErrors=False)

  from LHCbDIRAC.BookkeepingSystem.Client.ScriptExecutors import executeGetStats
  executeGetStats(dmScript)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
""" Collection of user jobs for testing purposes
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import time
import errno

from DIRAC.DataManagementSystem.Utilities.DMSHelpers import DMSHelpers
from DIRAC.tests.Utilities.testJobDefinitions import baseToAllJobs, endOfAllJobs, find_all
from DIRAC.Core.Utilities.Proxy import executeWithUserProxy
from LHCbDIRAC.Interfaces.API.LHCbJob import LHCbJob
from LHCbDIRAC.Interfaces.API.DiracLHCb import DiracLHCb
from LHCbDIRAC.tests.Workflow import createJob

from LHCbDIRAC import rootPath

# parameters

jobClass = LHCbJob
diracClass = DiracLHCb

try:
  tier1s = DMSHelpers().getTiers(tier=(0, 1))
except AttributeError:
  tier1s = ['LCG.CERN.cern', 'LCG.CNAF.it', 'LCG.GRIDKA.de', 'LCG.IN2P3.fr',
            'LCG.NIKHEF.nl', 'LCG.PIC.es', 'LCG.RAL.uk', 'LCG.RRCKI.ru', 'LCG.SARA.nl']

# List of jobs
wdir = os.getcwd()


@executeWithUserProxy
def helloWorldTestT2s():

  job = baseToAllJobs('helloWorldTestT2s', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setBannedSites(tier1s)
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestCERN():

  job = baseToAllJobs('helloWorld-test-CERN', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination('LCG.CERN.cern')
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestIN2P3():
  job = baseToAllJobs('helloWorld-test-IN2P3', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination('LCG.IN2P3.fr')
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestGRIDKA():
  job = baseToAllJobs('helloWorld-test-GRIDKA', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination('LCG.GRIDKA.de')
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestARC():

  job = baseToAllJobs('helloWorld-test-ARC', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination(['LCG.RAL.uk'])
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestSSH():

  job = baseToAllJobs('helloWorld-test-SSH', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination(['DIRAC.Zurich.ch'])
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestSSHCondor():

  job = baseToAllJobs('helloWorld-test-SSHCondor', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination(['DIRAC.Syracuse.us'])  # FIXME: not there
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestVAC():

  job = baseToAllJobs('helloWorld-test-VAC', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination(['VAC.Manchester.uk'])  # FIXME: not there
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestCLOUD():

  job = baseToAllJobs('helloWorld-test-CLOUD', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setDestination(['CLOUD.CERN.cern'])
  return endOfAllJobs(job)


@executeWithUserProxy
def helloWorldTestCentos7():

  job = baseToAllJobs('helloWorld-test-CentOS7', jobClass)
  job.setInputSandbox([find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setPlatform('x86_64+avx2+fma-centos7-gcc8-opt')
  return endOfAllJobs(job)


@executeWithUserProxy
def jobWithOutput():

  timenow = time.strftime("%s")
  with open(os.path.join(wdir, timenow + "testFileUpload.txt"), "w") as f:
    f.write(timenow)
  inp1 = [find_all(timenow + 'testFileUpload.txt', '.')[0]]
  inp2 = [find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]]
  job = baseToAllJobs('jobWithOutput', jobClass)
  job.setInputSandbox(inp1 + inp2)
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setOutputData([timenow + 'testFileUpload.txt'])
  res = endOfAllJobs(job)
  try:
    os.remove(os.path.join(wdir, timenow + "testFileUpload.txt"))
  except OSError as e:
    return e.errno == errno.ENOENT
  return res


@executeWithUserProxy
def jobWithOutputAndPrepend():

  timenow = time.strftime("%s")
  with open(os.path.join(wdir, timenow + "testFileUploadNewPath.txt"), "w") as f:
    f.write(timenow)
  job = baseToAllJobs('jobWithOutputAndPrepend', jobClass)
  inp1 = [find_all(timenow + 'testFileUploadNewPath.txt', '.')[0]]
  inp2 = [find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]]
  job.setInputSandbox(inp1 + inp2)
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setOutputData([timenow + 'testFileUploadNewPath.txt'], filePrepend='testFilePrepend')
  res = endOfAllJobs(job)
  try:
    os.remove(os.path.join(wdir, timenow + "testFileUploadNewPath.txt"))
  except OSError as e:
    return e.errno == errno.ENOENT
  return res


@executeWithUserProxy
def jobWithOutputAndPrependWithUnderscore():

  timenow = time.strftime("%s")
  with open(os.path.join(wdir, timenow + "testFileUpload_NewPath.txt"), "w") as f:
    f.write(timenow)
  job = baseToAllJobs('jobWithOutputAndPrependWithUnderscore', jobClass)
  inp1 = [find_all(timenow + 'testFileUpload_NewPath.txt', '.')[0]]
  inp2 = [find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]]
  job.setInputSandbox(inp1 + inp2)
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  res = job.setOutputData([timenow + 'testFileUpload_NewPath.txt'], filePrepend='testFilePrepend')
  if not res['OK']:
    return 0
  res = endOfAllJobs(job)
  try:
    os.remove(os.path.join(wdir, timenow + 'testFileUpload_NewPath.txt'))
  except OSError as e:
    return e.errno == errno.ENOENT
  return res


@executeWithUserProxy
def jobWithOutputAndReplication():

  timenow = time.strftime("%s")
  with open(os.path.join(wdir, timenow + "testFileReplication.txt"), "w") as f:
    f.write(timenow)
  job = baseToAllJobs('jobWithOutputAndReplication', jobClass)
  inp1 = [find_all(timenow + 'testFileReplication.txt', '.')[0]]
  inp2 = [find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]]
  job.setInputSandbox(inp1 + inp2)
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setOutputData([timenow + 'testFileReplication.txt'], replicate='True')
  res = endOfAllJobs(job)
  try:
    os.remove(os.path.join(wdir, timenow + 'testFileReplication.txt'))
  except OSError as e:
    return e.errno == errno.ENOENT
  return res


@executeWithUserProxy
def jobWith2OutputsToBannedSE():

  timenow = time.strftime("%s")
  with open(os.path.join(wdir, timenow + "testFileUploadBanned-1.txt"), "w") as f:
    f.write(timenow)
  with open(os.path.join(wdir, timenow + "testFileUploadBanned-2.txt"), "w") as f:
    f.write(timenow)
  job = baseToAllJobs('jobWith2OutputsToBannedSE', jobClass)
  inp1 = [find_all(timenow + 'testFileUploadBanned-1.txt', '.')[0]]
  inp2 = [find_all(timenow + 'testFileUploadBanned-2.txt', '.')[0]]
  inp3 = [find_all('exe-script.py', rootPath, '/tests/System/GridTestSubmission')[0]]
  inp4 = [find_all('partialConfig.cfg', rootPath, '/tests/System/GridTestSubmission')[0]]
  job.setInputSandbox(inp1 + inp2 + inp3 + inp4)
  job.setExecutable("exe-script.py", "", "helloWorld.log")
  job.setConfigArgs('partialConfig.cfg')
  job.setDestination('LCG.PIC.es')
  job.setOutputData(
      [timenow + 'testFileUploadBanned-1.txt', timenow + 'testFileUploadBanned-2.txt'], OutputSE=['PIC-USER'])
  res = endOfAllJobs(job)
  try:
    os.remove(os.path.join(wdir, timenow + 'testFileUploadBanned-1.txt'))
  except OSError as e:
    return e.errno == errno.ENOENT
  try:
    os.remove(os.path.join(wdir, timenow + 'testFileUploadBanned-2.txt'))
  except OSError as e:
    return e.errno == errno.ENOENT
  return res


@executeWithUserProxy
def jobWithSingleInputData():

  job = baseToAllJobs('jobWithSingleInputData-shouldGoToCERN', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('download')
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataCERN():

  job = baseToAllJobs('jobWithSingleInputDataCERN-shouldSucceed', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('download')
  job.setDestination(['LCG.CERN.cern'])
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataRAL():

  job = baseToAllJobs('jobWithSingleInputDataRAL-shouldFailOptimizers', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('protocol')
  job.setDestination(['LCG.RAL.uk'])
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataIN2P3():

  job = baseToAllJobs('jobWithSingleInputDataIN2P3-shouldFailOptimizers', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('protocol')
  job.setDestination(['LCG.IN2P3.fr'])
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataRRCKI():

  job = baseToAllJobs('jobWithSingleInputDataRRCKI-shouldFailOptimizers', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('protocol')
  job.setDestination(['LCG.RRCKI.ru'])
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataSARA():

  job = baseToAllJobs('jobWithSingleInputDataSARA-shouldFailOptimizers', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('protocol')
  job.setDestination(['LCG.SARA.nl'])
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataPIC():

  job = baseToAllJobs('jobWithSingleInputDataPIC-shouldFailOptimizers', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-single-location.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-single-location.py", "", "exeWithInput.log")
  # this file should be at CERN-USER only
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFileSingleLocation.txt')
  job.setInputDataPolicy('protocol')
  job.setDestination(['LCG.PIC.es'])
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithSingleInputDataSpreaded():

  job = baseToAllJobs('jobWithSingleInputDataSpreaded', jobClass)
  job.setInputSandbox([find_all('exe-script-with-input.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input.py", "", "exeWithInput.log")
  # this file should be at CERN-USER and IN2P3-USER
  job.setInputData('/lhcb/user/f/fstagni/test/testInputFile.txt')
  job.setInputDataPolicy('download')
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def jobWithInputDataAndAncestor():

  job = baseToAllJobs('jobWithInputDataAndAncestor', jobClass)
  job.setInputSandbox(
      [find_all('exe-script-with-input-and-ancestor.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setExecutable("exe-script-with-input-and-ancestor.py", "", "exeWithInput.log")
  # WARNING: Collision10!!
  job.setInputData('/lhcb/data/2010/SDST/00008375/0005/00008375_00053941_1.sdst')  # this file should be at SARA-RDST
  # the ancestor should be /lhcb/data/2010/RAW/FULL/LHCb/COLLISION10/81616/081616_0000000213.raw (CERN and SARA)
  job.setAncestorDepth(1)  # pylint: disable=no-member
  job.setInputDataPolicy('download')
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def gaussJob():

  job = baseToAllJobs('gaussJob', jobClass)
  job.setInputSandbox(
      [find_all('prodConf_Gauss_00012345_00067890_1.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setOutputSandbox('00012345_00067890_1.sim')

  optGauss = "$APPCONFIGOPTS/Gauss/Sim08-Beam3500GeV-md100-2011-nu2.py;"
  optDec = "$DECFILESROOT/options/34112104.py;"
  optPythia = "$LBPYTHIAROOT/options/Pythia.py;"
  optOpts = "$APPCONFIGOPTS/Gauss/G4PL_FTFP_BERT_EmNoCuts.py;"
  optCompr = "$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py;"
  optPConf = "prodConf_Gauss_00012345_00067890_1.py"
  options = optGauss + optDec + optPythia + optOpts + optCompr + optPConf
  job.setApplication('Gauss', 'v45r5', options,  # pylint: disable=no-member
                     extraPackages='AppConfig.v3r179;Gen/DecFiles.v27r14p1;ProdConf.v1r9',
                     systemConfig='x86_64-slc5-gcc43-opt')
  job.setDIRACPlatform()  # pylint: disable=no-member
  job.setCPUTime(172800)
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def booleJob():

  job = baseToAllJobs('booleJob', jobClass)
  job.setInputSandbox(
      [find_all('prodConf_Boole_00012345_00067890_1.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setOutputSandbox('00012345_00067890_1.digi')

  opts = "$APPCONFIGOPTS/Boole/Default.py;"
  optDT = "$APPCONFIGOPTS/Boole/DataType-2011.py;"
  optTCK = "$APPCONFIGOPTS/Boole/Boole-SiG4EnergyDeposit.py;"
  optComp = "$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py;"
  optPConf = "prodConf_Boole_00012345_00067890_1.py"
  options = opts + optDT + optTCK + optComp + optPConf

  job.setApplication('Boole', 'v26r3', options,  # pylint: disable=no-member
                     inputData='/lhcb/user/f/fstagni/test/12345/12345678/00012345_00067890_1.sim',
                     extraPackages='AppConfig.v3r171;ProdConf.v1r9',
                     systemConfig='x86_64-slc5-gcc43-opt')

  job.setDIRACPlatform()  # pylint: disable=no-member
  job.setCPUTime(172800)
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def booleJobWithConf():

  job = baseToAllJobs('booleJobWithConf', jobClass)
  job.setInputSandbox([find_all('prodConf_Boole_00012345_00067890_1.py', rootPath,
                                '/tests/System/GridTestSubmission')[0],
                       find_all('rootConfig.cfg', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setOutputSandbox('00012345_00067890_1.digi')

  opts = "$APPCONFIGOPTS/Boole/Default.py;"
  optDT = "$APPCONFIGOPTS/Boole/DataType-2011.py;"
  optTCK = "$APPCONFIGOPTS/Boole/Boole-SiG4EnergyDeposit.py;"
  optComp = "$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py;"
  optPConf = "prodConf_Boole_00012345_00067890_1.py"
  options = opts + optDT + optTCK + optComp + optPConf

  job.setApplication('Boole', 'v26r3', options,  # pylint: disable=no-member
                     inputData='/lhcb/user/f/fstagni/test/12345/12345678/00012345_00067890_1.sim',
                     extraPackages='AppConfig.v3r171;ProdConf.v1r9',
                     systemConfig='x86_64-slc5-gcc43-opt')

  job.setDIRACPlatform()  # pylint: disable=no-member
  job.setConfigArgs('rootConfig.cfg')
  job.setCPUTime(172800)
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def gaudiApplicationScriptJob():

  job = baseToAllJobs('gaudiApplicationScriptJob', jobClass)
  inp1 = [find_all('_input_sandbox_1324_master.tgz', wdir, 'tests/System/GridTestSubmission')[0]]
  inp2 = [find_all('runToys.C', wdir, 'tests/System/GridTestSubmission')[0]]
  inp3 = [find_all('script_wrapper.py', wdir, 'tests/System/GridTestSubmission')[0]]
  job.setInputSandbox(inp1 + inp2 + inp3)
  executable = find_all('script_wrapper.py',
                        wdir,
                        'tests/System/GridTestSubmission/script_wrapper.py')[0]  # pylint: disable=no-member
  job.setApplicationScript('root', '6.06.02',
                           executable,
                           systemConfig='x86_64-slc6-gcc49-opt')
  job.setOutputSandbox('FitResultsToyData*.root')
  # job.setDIRACPlatform()  # pylint: disable=no-member
  job.setCPUTime(172800)
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def wrongJob():

  print("\n Submitting gaudiRun job (Gauss only) that will use a configuration file that contains wrong info")
  print("This will generate a job that should become Completed, use the failover, and only later it will be Done")

  job = createJob(local=False)
  job.setName("gaudirun-gauss-stays-completed")
  res = endOfAllJobs(job)
  return res


@executeWithUserProxy
def gaussMPJob():

  job = baseToAllJobs('GaussMP_v49r14', jobClass)

  job.setInputSandbox(
      [find_all('prodConf_Gauss_MP_test.py', rootPath, '/tests/System/GridTestSubmission')[0]])
  job.setOutputSandbox('Gauss_MP_test.sim')

  options = "$APPCONFIGOPTS/Gauss/Beam6500GeV-mu100-2018-nu1.6.py;"
  options += "$APPCONFIGOPTS/Gauss/EnableSpillover-25ns.py;"
  options += "$APPCONFIGOPTS/Gauss/DataType-2017.py"
  options += "$APPCONFIGOPTS/Gauss/RICHRandomHits.py"
  options += "$DECFILESROOT/options/10132060.py"
  options += "$LBPYTHIA8ROOT/options/Pythia8.py"
  options += "$APPCONFIGOPTS/Gauss/G4PL_FTFP_BERT_EmNoCuts.py"
  job.setApplication('Gauss', 'v49r14', options,  # pylint: disable=no-member
                     extraPackages='AppConfig.v3r383;Gen/DecFiles.v30r32;ProdConf.v2r8',
                     systemConfig='x86_64-slc6-gcc48-opt')
  job.setDIRACPlatform()  # pylint: disable=no-member
  job.setCPUTime(172800)
  job.setNumberOfProcessors(4)

  return endOfAllJobs(job)



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import sys
import re
import fileinput
import json
from diraccfg import parseVersion, CFG


def linePrepend(filename, line):
  with open(filename, 'r+') as f:
    content = f.read()
    f.seek(0, 0)
    f.write(line.rstrip('\r\n') + '\n' + content)


def lineReplace(filename, lineToReplace, newLine):
  for line in fileinput.input(filename, inplace=True):
    if lineToReplace == 0:
      print(newLine)
    else:
      print(line, end="")
    lineToReplace -= 1


# Load version from last release and get last version
LATEST_RELEASE = os.environ.get('LATEST_RELEASE')

# Read in the releases.cfg as JSON
res = CFG().loadFromFile("releases.cfg").getAsDict()

# From the releases.cfg look up the versions of DIRAC, LHCbWebDIRAC, LHCbDIRACOS used in last version
LAST_DIRAC = res['Releases'][LATEST_RELEASE]['Depends'].split(':', 1)[-1]
LAST_LHCbDIRACOS = res['Releases'][LATEST_RELEASE]['DIRACOS'].split(':', 1)[-1]
LAST_LHCbWebDIRAC = res['Releases'][LATEST_RELEASE]['Modules'].split(':', 2)[-1]

# Read env variables defined by user or use version from previous release
DIRAC = os.getenv('DIRAC', LAST_DIRAC)
LHCbDIRACOS = os.getenv('LHCbDIRACOS', LAST_LHCbDIRACOS)
LHCbWebDIRAC = os.getenv('LHCbWebDIRAC', LAST_LHCbWebDIRAC)

# Check if user specified version for next release
NEXT_RELEASE = os.environ.get('NEXT_RELEASE')

# If the user did not specify a release increment the current version by 1
version = None
versionString = None
versionStringPy3 = None
preRelease = None
if not NEXT_RELEASE:
  version = parseVersion(LATEST_RELEASE)
  if version[3] is None:
    # Increment patch version for 1
    version = (version[0], version[1], version[2] + 1, version[3])
    versionString = "v%sr%sp%s" % (version[0], version[1], version[2])
    versionStringPy3 = "v%s.%s.%s" % (version[0], version[1], version[2])
    preRelease = False
    print("Automatically increment current release %s to %s" % (LATEST_RELEASE, versionString))
  else:
    # Increment pre version for 1
    version = (version[0], version[1], version[2], version[3] + 1)
    versionString = "v%sr%s-pre%s" % (version[0], version[1], version[3])
    versionStringPy3 = "v%s.%s.0a%s" % (version[0], version[1], version[3])
    preRelease = True
    print("Automatically increment current release %s to %s" % (LATEST_RELEASE, versionString))
else:
  # Use the version specified by NEXT_RELEASE
  version = parseVersion(NEXT_RELEASE)
  print("Preparing files for release %s" % NEXT_RELEASE)
  if version[3] is None:
    preRelease = False
  else:
    preRelease = True

# Construct the new releases section
newCFG = "\n  %s\n  {\n    Modules = LHCbDIRAC:%s, LHCbWebDIRAC:%s\n    " \
    "Depends = DIRAC:%s\n    DIRACOS = LHCb:%s\n  }\n" % \
    (versionString, versionString, LHCbWebDIRAC, DIRAC, LHCbDIRACOS)
print("I am adding the following section to releases.cfg")
print(newCFG)

# Insert into releases.cfg the new release
lineReplace("releases.cfg", 22, newCFG)

# Add information about version to release notes
linePrepend("../notes.txt", "LHCbDIRACOS %s" % LHCbDIRACOS)
linePrepend("../notes.txt", "LHCbWebDIRAC %s" % LHCbWebDIRAC)
linePrepend("../notes.txt", "Based on DIRAC %s" % DIRAC)
linePrepend("../notes.txt", versionString)


# store into artifact the version and series for later usage in tagging process
with open('version.txt', 'a') as fver:
  fver.write(versionString)
if versionStringPy3:
  with open('versionPy3.txt', 'a') as fver:
    fver.write(versionStringPy3)
with open('series.txt', 'a') as fser:
  fser.write("v%sr%s" % (version[0], version[1]))

# change the __init__.py
# Currently master and devel have different layout
if not preRelease:
  lineReplace("src/LHCbDIRAC/__init__.py", 43, "  majorVersion = %s" % version[0])
  lineReplace("src/LHCbDIRAC/__init__.py", 44, "  minorVersion = %s" % version[1])
  lineReplace("src/LHCbDIRAC/__init__.py", 45, "  patchLevel = %s" % version[2])
  lineReplace("src/LHCbDIRAC/__init__.py", 46, "  preVersion = %s" % 0)
else:
  lineReplace("src/LHCbDIRAC/__init__.py", 43, "  majorVersion = %s" % version[0])
  lineReplace("src/LHCbDIRAC/__init__.py", 44, "  minorVersion = %s" % version[1])
  lineReplace("src/LHCbDIRAC/__init__.py", 45, "  patchLevel = %s" % 0)
  lineReplace("src/LHCbDIRAC/__init__.py", 46, "  preVersion = %s" % version[3])



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Base Entity System client."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from LHCbDIRAC.BookkeepingSystem.Client.BaseESManager import BaseESManager


__RCSID__ = "$Id$"


class BaseESClient(object):
  """Basic client."""

  #############################################################################
  def __init__(self, esManager=BaseESManager(), path="/"):
    """The Entity manager must be initialized which will be used to manipulate
    the databaase."""
    self.__ESManager = esManager
    result = self.getManager().getAbsolutePath(path)
    if result['OK']:
      self.__currentDirectory = result['Value']

  #############################################################################
  def list(self, path="", selectionDict=None, sortDict=None, startItem=0, maxitems=0):
    """It lists the database content as a Linux File System."""
    selectionDict = selectionDict if selectionDict is not None else {}
    sortDict = sortDict if sortDict is not None else {}
    res = self.getManager().mergePaths(self.__currentDirectory, path)
    if res['OK']:
      return self.getManager().list(res['Value'], selectionDict, sortDict, startItem, maxitems)
    return res

  #############################################################################
  def getManager(self):
    """It returns the manager whicg used to manipulate the database."""
    return self.__ESManager

  #############################################################################
  def get(self, path=""):
    """It return the actual directory."""
    return self.getManager().get(path)

  #############################################################################
  def getPathSeparator(self):
    """It returns the space separator."""
    return self.getManager().getPathSeparator()

  #############################################################################



#! /usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Set the status (default Unused) of a list of LFNs or files in status
<Status> of Transformation <TransID>"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import DIRAC

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
  from DIRAC.Core.Utilities.List import breakListIntoChunks

  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, Script
  from LHCbDIRAC.TransformationSystem.Utilities.ScriptUtilities import getTransformations

  dmScript = DMScript()
  dmScript.registerFileSwitches()
  newStatus = 'Unused'
  statusList = ("Unused", "Assigned", "Done", "Problematic", "MissingLFC", "MissingInFC", "MaxReset",
                "Processed", "NotProcessed", "Removed", 'ProbInFC')
  Script.registerSwitch('', 'Status=', "Select files with a given status from %s" % str(statusList))
  Script.registerSwitch('', 'NewStatus=', "New status to be set (default: %s)" % newStatus)
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] <TransID> <Status>' % Script.scriptName, ]))
  Script.parseCommandLine()

  switches = Script.getUnprocessedSwitches()
  status = None
  for opt, val in switches:
    if opt == 'Status':
      val = set(val.split(','))
      if val & set(statusList) != val:
        print("Unknown status %s... Select in %s" % (','.join(val), str(statusList)))
        Script.showHelp(exitCode=1)
      status = list(val)
    elif opt == 'NewStatus':
      if val not in statusList:
        print("Unknown status %s... Select in %s" % (val, str(statusList)))
        Script.showHelp()
        DIRAC.exit(1)
      newStatus = val

  args = Script.getPositionalArgs()
  idList = getTransformations([args[0]])
  if not idList:
    DIRAC.exit(1)

  if len(args) == 2:
    status = args[1].split(',')
  elif not status:
    status = ['Unknown']
  lfnsExplicit = dmScript.getOption('LFNs')

  transClient = TransformationClient()

  for transID in idList:
    lfns = lfnsExplicit
    if not lfns:
      res = transClient.getTransformation(transID)
      if not res['OK']:
        print("Failed to get transformation information: %s" % res['Message'])
        DIRAC.exit(2)

      selectDict = {'TransformationID': res['Value']['TransformationID'], 'Status': status}
      res = transClient.getTransformationFiles(condDict=selectDict)
      if not res['OK']:
        print("Failed to get files: %s" % res['Message'])
        DIRAC.exit(2)

      lfns = [d['LFN'] for d in res['Value']]
      if not lfns:
        print("No files found in transformation %s, status %s" % (transID, status))

    if not lfns:
      print("No files to be set in transformation", transID)
    else:
      resetFiles = 0
      failed = {}
      for lfnChunk in breakListIntoChunks(lfns, 10000):
        force = 'MaxReset' in status or 'Processed' in status or lfnsExplicit
        res = transClient.setFileStatusForTransformation(transID, newStatus, lfnChunk,
                                                         force=force)
        if res['OK']:
          resetFiles += len(res['Value'].get('Successful', res['Value']))
          for lfn, reason in res['Value'].get('Failed', {}).items():
            if reason != 'File not found in the Transformation Database':
              failed.setdefault(reason, []).append(lfn)
        else:
          print("Failed to set %d files to %s in transformation %s: %s" %
                (len(lfns), newStatus, transID, res['Message']))
      print("%d files were set %s in transformation %s" % (resetFiles, newStatus, transID))
      if failed:
        for reason in failed:
          print('Failed for %d files: %s' % (len(failed[reason]), reason))
  DIRAC.exit(0)


if __name__ == "__main__":
  main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Set Data Quality Flag for the given run."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script

  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] ... Run Flag' % Script.scriptName,
      'Arguments:',
      '  Run:      Run number',
      '  Flag:     Quality Flag']))
  Script.parseCommandLine(ignoreErrors=True)
  args = Script.getPositionalArgs()

  from LHCbDIRAC.BookkeepingSystem.Client.BookkeepingClient import BookkeepingClient
  bk = BookkeepingClient()

  if len(args) < 2:
    result = bk.getAvailableDataQuality()
    if not result['OK']:
      print('ERROR: %s' % (result['Message']))
      DIRAC.exit(2)
    flags = result['Value']
    print("Available Data Quality Flags")
    for flag in flags:
      print(flag)
    Script.showHelp()

  exitCode = 0
  rnb = int(args[0])
  flag = args[1]
  result = bk.setRunDataQuality(rnb, flag)

  if not result['OK']:
    print('ERROR: %s' % (result['Message']))
    exitCode = 2
  else:
    succ = result['Value']['Successful']
    failed = result['Value']['Failed']
    print('The data quality has been set for the following files:')
    for i in succ:
      print(i)

    if len(failed) != 0:
      print('The data quality has not been set for the following files:')
      for i in failed:
        print(i)

  DIRAC.exit(exitCode)


if __name__ == "__main__":
  main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Check if the files are in the BK, the FC and the SEs they are supposed to be
in.

Uses the DM script switches, and, unless a list of LFNs is provided

1) If --Directory is used: get files in FC directories
2) If --Production or --BK options is used get files in the FC directories from the BK

If an (list of) SE or SE-group is specified option --SE) and the file is not in the FC,
check if it is in these SEs

- If the file is in the SE and in the BK, it can be registered in the FC using --FixIt

If --FixIt is set, takes actions:
  Missing files: remove from SE and FC
  No replica flag: set it (in the BK)
  Not existing in SE: remove replica or file from the catalog
  Existing in SE and BK: register it in the FC
  Bad checksum: remove replica or file from SE and catalogs if no good replica
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


def __getSEsFromOptions(dmScript):
  from DIRAC.DataManagementSystem.Utilities.DMSHelpers import DMSHelpers
  seList = dmScript.getOption('SEs', [])
  sites = dmScript.getOption('Sites', [])
  if sites:
    siteSEs = []
    dmsHelper = DMSHelpers()
    for site in sites:
      siteSEs += dmsHelper.getSEsForSite(site).get('Value', [])
    if seList and siteSEs:
      seList = list(set(seList) & set(siteSEs))
    else:
      seList += siteSEs
  return seList


@DIRACScript()
def main():
  # Script initialization
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, Script

  Script.setUsageMessage('\n'.join([__doc__,
                                    'Usage:',
                                    '  %s [option|cfgfile] [values]' % Script.scriptName, ]))
  dmScript = DMScript()
  dmScript.registerDMSwitches()
  maxFiles = 20
  Script.registerSwitch('', 'FixIt', '   Take action to fix the catalogs and storage')
  Script.registerSwitch('', 'Replace', '   Replace bad or missing replicas (default=False)')
  Script.registerSwitch('', 'NoBK', '   Do not check with BK')
  Script.registerSwitch('', 'Verbose', '   Set logging mode to INFO')
  Script.registerSwitch('', 'MaxFiles=', '   Set maximum number of files to be printed (default %d)' % maxFiles)
  Script.parseCommandLine(ignoreErrors=True)

  from DIRAC import gLogger

  fixIt = False
  bkCheck = True
  replace = False
  verbose = False
  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'FixIt':
      fixIt = True
    elif switch[0] == 'NoBK':
      bkCheck = False
    elif switch[0] == 'Replace':
      replace = True
    elif switch[0] == 'Verbose':
      verbose = True
    elif switch[0] == 'MaxFiles':
      try:
        maxFiles = int(switch[1])
      except Exception as e:
        gLogger.exception("Invalid value for MaxFiles", lException=e)
        pass

  # imports
  from LHCbDIRAC.DataManagementSystem.Client.ConsistencyChecks import ConsistencyChecks
  if verbose:
    gLogger.setLevel('INFO')
  cc = ConsistencyChecks()
  cc.directories = dmScript.getOption('Directory', [])
  cc.lfns = dmScript.getOption('LFNs', []) + [lfn for arg in Script.getPositionalArgs() for lfn in arg.split(',')]
  bkQuery = dmScript.getBKQuery(visible='All')
  if bkQuery:
    bkQuery.setOption('ReplicaFlag', 'All')
    cc.bkQuery = bkQuery

  cc.seList = __getSEsFromOptions(dmScript)
  from LHCbDIRAC.DataManagementSystem.Client.CheckExecutors import doCheckFC2SE
  try:
    doCheckFC2SE(cc, bkCheck=bkCheck, fixIt=fixIt, replace=replace, maxFiles=maxFiles)
  except RuntimeError as e:
    gLogger.fatal(str(e))
  except Exception as e:
    gLogger.exception('Exception', lException=e)


if __name__ == "__main__":
  main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Remove replicas of a (list of) LFNs at a list of sites.

It is possible to request a minimum of remaining replicas
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, Script

  dmScript = DMScript()
  dmScript.registerFileSwitches()
  dmScript.registerSiteSwitches()

  Script.registerSwitch("n", "NoFC", " use this option to force the removal from storage of replicas not in FC")
  Script.registerSwitch(
      '',
      'ReduceReplicas=',
      '  specify the number of replicas you want to keep (default SE: Tier1-USER)')
  Script.registerSwitch("", "Force", " use this option for force the removal of replicas even if last one")
  Script.setUsageMessage('\n'.join([__doc__,
                                    'Usage:',
                                    '  %s [option|cfgfile] ... [LFN[,LFN2[,LFN3...]]] SE[,SE2...]' % Script.scriptName,
                                    'Arguments:',
                                    '  LFN:      Logical File Name or file containing LFNs',
                                    '  SE:       Valid DIRAC SE']))
  Script.parseCommandLine()

  from LHCbDIRAC.DataManagementSystem.Client.ScriptExecutors import executeRemoveReplicas
  from DIRAC import exit
  exit(executeRemoveReplicas(dmScript))


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""This is the LHCb Online storage."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from six.moves import xmlrpc_client as xmlrpclib
import os

from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Resources.Storage.StorageBase import StorageBase

from DIRAC.Resources.Storage.Utilities import checkArgumentFormat
from DIRAC.Core.Utilities.List import breakListIntoChunks

__RCSID__ = "$Id$"


class LHCbOnlineStorage(StorageBase):
  """Plugin to talk to the xmlrpc of the datamover."""

  def __init__(self, storageName, parameterDict):
    self.isok = True

    super(LHCbOnlineStorage, self).__init__(storageName, parameterDict)
    self.pluginName = 'LHCbOnline'
    self.name = storageName
    self.timeout = 100

    serverString = "%s://%s:%s" % (self.protocolParameters['Protocol'],
                                   self.protocolParameters['Host'], self.protocolParameters['Port'])
    self.server = xmlrpclib.Server(serverString)

  def getFileSize(self, urls):
    # FIXME: What the hell is this method doing ??
    """Get a fake file size."""
    if not urls:
      return S_ERROR("LHCbOnline.getFileSize: No surls supplied.")
    successful = {}
    failed = {}
    for pfn in urls:
      successful[pfn] = 0
    resDict = {'Failed': failed, 'Successful': successful}
    return S_OK(resDict)

  def retransferOnlineFile(self, urls):
    """Tell the Online system that the migration failed and we want to get the
    request again."""

    if not urls:
      return S_ERROR("LHCbOnline.requestRetransfer: No surls supplied.")
    successful = {}
    failed = {}
    for pfn in urls:
      try:
        success, error = self.server.errorMigratingFile(pfn)
        if success:
          successful[pfn] = True
          gLogger.info("LHCbOnline.requestRetransfer: Successfully requested file from RunDB.")
        else:
          errStr = "LHCbOnline.requestRetransfer: Failed to request file from RunDB: %s" % error
          failed[pfn] = errStr
          gLogger.error(errStr, pfn)
      except Exception as x:  # pylint: disable=broad-except
        errStr = "LHCbOnline.requestRetransfer: Exception while requesting file from RunDB."
        gLogger.exception(errStr, lException=x)
        failed[pfn] = errStr
    resDict = {'Failed': failed, 'Successful': successful}
    return S_OK(resDict)

  def removeFile(self, urls):
    """Remove physically the file specified by its path."""
    if not urls:
      return S_ERROR("LHCbOnline.removeFile: No surls supplied.")
    successful = {}
    failed = {}
    # Here we are sure of the unicity of the basename since it is for raw data only
    filesToUrls = dict((os.path.basename(f), f) for f in urls)
    filenames = list(filesToUrls)

    for filenameChunck in breakListIntoChunks(filenames, 100):

      try:
        success, errorOrFailed = self.server.endMigratingFileBulk(filenameChunck)
        if success:
          # in case of success, errorOrFailed contains the files for which it failed
          failedFiles = set(errorOrFailed)
          for fn in filenameChunck:
            fullUrl = filesToUrls[fn]
            if fn in failedFiles:
              failed[fullUrl] = "Failed to remove, check datamover logs"
            else:
              successful[fullUrl] = True
          gLogger.info("LHCbOnline.getFile: Successfully issued removal to RunDB for chuck.")
        else:
          errStr = "LHCbOnline.removeFile: Failed to issue removal for chunck to RunDB %s" % errorOrFailed
          for fn in filenameChunck:
            fullUrl = filesToUrls[fn]
            failed[fullUrl] = errStr
          gLogger.error(errStr, urls)
      except Exception as x:  # pylint: disable=broad-except
        errStr = "LHCbOnline.getFile: Exception for chunck while issuing removal to RunDB."
        gLogger.exception(errStr, lException=x)
        for fn in filenameChunck:
          fullUrl = filesToUrls[fn]
          failed[fullUrl] = errStr

    resDict = {'Failed': failed, 'Successful': successful}
    return S_OK(resDict)



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Unittest for: LHCbDIRAC.AccountingSystem.private.Plotters.StoragePlotter.

StoragePlotter.__bases__:
  DIRAC.AccountingSystem.private.Plotters.BaseReporter

We are assuming there is a solid test of __bases__, we are not testing them
here and assuming they work fine.

IMPORTANT: the test MUST be pylint compliant !
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=protected-access

import unittest
from decimal import Decimal

import mock


class StoragePlotterTestCase(unittest.TestCase):
  """StoragePlotterTestCase."""

  moduleTested = None
  classsTested = None

  def mockModuleTested(self, moduleTested):
    """Used to not redo the mocking done on the parent class ( if any )"""

    # Tries to get the mocks of the parent TestCases ( if any )
    for baseClass in self.__class__.__bases__:
      try:
        # pylint: disable=no-member
        moduleTested = baseClass.mockModuleTested(moduleTested)
      except TypeError:
        continue

    # And then makes its own mock
    class MockStorage(object):
      # pylint: disable=missing-docstring,no-init
      definitionKeyFields = ('StorageElement', 'Directory')

    moduleTested.Storage = mock.Mock(return_value=MockStorage())

    return moduleTested

  def setUp(self):
    """Setup the test case."""

    import LHCbDIRAC.AccountingSystem.private.Plotters.StoragePlotter as moduleTested

    self.moduleTested = self.mockModuleTested(moduleTested)
    self.classsTested = self.moduleTested.StoragePlotter

  def tearDown(self):
    """Tear down the test case."""

    del self.moduleTested
    del self.classsTested

# ...............................................................................


class StoragePlotterUnitTest(StoragePlotterTestCase):
  """DataStoragePlotterUnitTest.

  <constructor>
   - test_instantiate
  <class variables>
   - test_typeName
   - test_typeKeyFields
   - test_reportCatalogSpaceName
   - test_reportCatalogFilesName
   - test_reportPhysicalSpaceName
   - test_reportPhysicalFilesName
   - test_reportPFNvsLFNFileMultiplicityName
   - test_reportPFNvsLFNSizeMultiplicityName
  <methods>
   - test_reportCatalogSpace
   - test_plotCatalogSpace
   - test_reportCatalogFiles
   - test_plotCatalogFiles
   - test_reportPhysicalSpace
   - test_plotPhysicalSpace
   - test_reportPhysicalFiles
   - test_plotPhysicalFiles
  """
  # FIXME: missing test_reportPFNvsLFNFileMultiplicity
  # FIXME: missing test_plotPFNvsLFNFileMultiplicity
  # FIXME: missing test_reportPFNvsLFNSizeMultiplicity
  # FIXME: missing test_plotPFNvsLFNSizeMultiplicity
  # FIXME: missing test_multiplicityReport

  def test_instantiate(self):
    """tests that we can instantiate one object of the tested class."""
    obj = self.classsTested(None, None)
    self.assertEqual('StoragePlotter', obj.__class__.__name__)

  def test_typeName(self):
    """test the class variable "_typeName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._typeName, "Storage")

  def test_typeKeyFields(self):
    """test the class variable "_typeKeyFields"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._typeKeyFields, ['StorageElement', 'Directory'])

  def test_reportCatalogSpaceName(self):
    """test the class variable "_reportCatalogSpaceName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCatalogSpaceName, "LFN size")

  def test_reportCatalogFilesName(self):
    """test the class variable "_reportCatalogFilesName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportCatalogFilesName, "LFN files")

  def test_reportPhysicalSpaceName(self):
    """test the class variable "_reportPhysicalSpaceName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPhysicalSpaceName, "PFN size")

  def test_reportPhysicalFilesName(self):
    """test the class variable "_reportPhysicalFilesName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPhysicalFilesName, "PFN files")

  def test_reportPFNvsLFNFileMultiplicityName(self):
    """test the class variable "_reportPFNvsLFNFileMultiplicityName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPFNvsLFNFileMultiplicityName, "PFN/LFN file ratio")

  def test_reportPFNvsLFNSizeMultiplicityName(self):
    """test the class variable "_reportPFNvsLFNSizeMultiplicityName"."""
    obj = self.classsTested(None, None)
    self.assertEqual(obj._reportPFNvsLFNSizeMultiplicityName, "PFN/LFN size ratio")

  def test_reportCatalogSpace(self):
    """test the method "_reportCatalogSpace"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

    res = obj._reportCatalogSpace({'grouping': 'StorageElement'})
    self.assertEqual(res['OK'], False, msg='Rejected StorageElement grouping')
    self.assertEqual(res['Message'], 'Grouping by storage element when requesting lfn info makes no sense')

#     res = obj._reportCatalogSpace( { 'grouping'       : 'NextToABeer',
#                                      'groupingFields' : [ 0, [ 'mehh' ], 'blah' ],
#                                      'startTime'      : 'startTime',
#                                      'endTime'        : 'endTime',
#                                      'condDict'       : {}
#                                     } )
#     self.assertEqual( res[ 'OK' ], False )
#     self.assertEqual( res[ 'Message' ], 'No connection' )

    # Changed mocked to run over different lines of code
    mockAccountingDB._getConnection.return_value = {'OK': True, 'Value': []}
    res = obj._reportCatalogSpace({'grouping': 'NextToABeer',
                                   'groupingFields': [0, ['mehh'], 'blah'],
                                   'startTime': 'startTime',
                                   'endTime': 'endTime',
                                   'condDict': {}})
    self.assertEqual(res['OK'], True)
    self.assertEqual(res['Value'], {'graphDataDict': {},
                                    'data': {},
                                    'unit': 'MB',
                                    'granularity': 'BucketLength'})

    mockedData = (('/lhcb/data', 1355616000, 86400, Decimal('4935388524246.91')),
                  ('/lhcb/data', 1355702400, 86400, Decimal('4843844487074.82')),
                  ('/lhcb/LHCb', 1355616000, 86400, Decimal('3935388524246.91')),
                  ('/lhcb/LHCb', 1355702400, 86400, Decimal('3843844487074.82')))
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': mockedData}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 86400

    res = obj._reportCatalogSpace({'grouping': 'Directory',
                                   'groupingFields': ('%s', ['Directory']),
                                   'startTime': 1355663249.0,
                                   'endTime': 1355749690.0,
                                   'condDict': {'Directory': ['/lhcb/data', '/lhcb/LHCb']}
                                   })
    self.assertEqual(res['OK'], True)
    self.assertEqual(res['Value'], {'graphDataDict': {'/lhcb/data': {1355616000: 4.9353885242469104,
                                                                     1355702400: 4.8438444870748203},
                                                      '/lhcb/LHCb': {1355616000: 3.93538852424691,
                                                                     1355702400: 3.8438444870748198}
                                                      },
                                    'data': {'/lhcb/data': {1355616000: 4935388.5242469106,
                                                            1355702400: 4843844.4870748203},
                                             '/lhcb/LHCb': {1355616000: 3935388.5242469101,
                                                            1355702400: 3843844.4870748199}
                                             },
                                    'unit': 'TB',
                                    'granularity': 86400})

  def test_reportCatalogFiles(self):
    """test the method "_reportCatalogFiles"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

    res = obj._reportCatalogFiles({'grouping': 'StorageElement'})
    self.assertEqual(res['OK'], False)
    self.assertEqual(res['Message'], 'Grouping by storage element when requesting lfn info makes no sense')

#     res = obj._reportCatalogFiles( { 'grouping'       : 'NextToABeer',
#                                      'groupingFields' : [ 0, [ 'mehh' ], 'blah' ],
#                                      'startTime'      : 'startTime',
#                                      'endTime'        : 'endTime',
#                                      'condDict'       : {}
#                                     } )
#     self.assertEqual( res[ 'OK' ], False )
#     self.assertEqual( res[ 'Message' ], 'No connection' )

    # Changed mocked to run over different lines of code
    mockAccountingDB._getConnection.return_value = {'OK': True, 'Value': []}
    res = obj._reportCatalogFiles({'grouping': 'NextToABeer',
                                   'groupingFields': [0, ['mehh'], 'blah'],
                                   'startTime': 'startTime',
                                   'endTime': 'endTime',
                                   'condDict': {}
                                   })
    self.assertEqual(res['OK'], True, msg='Expected S_OK')
    self.assertEqual(res['Value'], {'graphDataDict': {},
                                    'data': {},
                                    'unit': 'files',
                                    'granularity': 'BucketLength'
                                    })

    mockedData = (('/lhcb/data', 1355616000, 86400, Decimal('4935388524246.91')),
                  ('/lhcb/data', 1355702400, 86400, Decimal('4843844487074.82')),
                  ('/lhcb/LHCb', 1355616000, 86400, Decimal('3935388524246.91')),
                  ('/lhcb/LHCb', 1355702400, 86400, Decimal('3843844487074.82')))
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': mockedData}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 86400

    res = obj._reportCatalogFiles({'grouping': 'Directory',
                                   'groupingFields': ('%s', ['Directory']),
                                   'startTime': 1355663249.0,
                                   'endTime': 1355749690.0,
                                   'condDict': {'Directory': ['/lhcb/data', '/lhcb/LHCb']}
                                   })
    self.assertEqual(res['OK'], True)
    self.assertEqual(res['Value'], {'graphDataDict': {'/lhcb/data': {1355616000: 4935388.5242469106,
                                                                     1355702400: 4843844.4870748203},
                                                      '/lhcb/LHCb': {1355616000: 3935388.5242469101,
                                                                     1355702400: 3843844.4870748199}
                                                      },
                                    'data': {'/lhcb/data': {1355616000: 4935388524246.9102,
                                                            1355702400: 4843844487074.8203},
                                             '/lhcb/LHCb': {1355616000: 3935388524246.9102,
                                                            1355702400: 3843844487074.8198}
                                             },
                                    'unit': 'Mfiles',
                                    'granularity': 86400
                                    })

  def test_reportPhysicalSpace(self):
    """test the method "_reportPhysicalSpace"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

#     res = obj._reportPhysicalSpace( { 'groupingFields' : [ 0, [ 'mehh' ], 'blah' ],
#                                       'startTime'      : 'startTime',
#                                       'endTime'        : 'endTime',
#                                       'condDict'       : {}
#                                      } )
#     self.assertEqual( res[ 'OK' ], False )
#     self.assertEqual( res[ 'Message' ], 'No connection' )

    # Changed mocked to run over different lines of code
    mockAccountingDB._getConnection.return_value = {'OK': True, 'Value': []}
    res = obj._reportPhysicalSpace({'grouping': 'NextToABeer',
                                    'groupingFields': [0, ['mehh'], 'blah'],
                                    'startTime': 'startTime',
                                    'endTime': 'endTime',
                                    'condDict': {}
                                    })
    self.assertEqual(res['OK'], True, msg='Expected S_OK')
    self.assertEqual(res['Value'], {'graphDataDict': {},
                                    'data': {},
                                    'unit': 'MB',
                                    'granularity': 'BucketLength'
                                    })

    mockedData = (('CERN-ARCHIVE', 1355616000, 86400, Decimal('2344556767812.91')),
                  ('CERN-ARCHIVE', 1355702400, 86400, Decimal('2544556767812.91')),
                  ('CERN-DST', 1355616000, 86400, Decimal('344556767812.91')),
                  ('CERN-DST', 1355702400, 86400, Decimal('544556767812.91')))
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': mockedData}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 86400

    res = obj._reportPhysicalSpace({'grouping': 'StorageElement',
                                    'groupingFields': ('%s', ['StorageElement']),
                                    'startTime': 1355663249.0,
                                    'endTime': 1355749690.0,
                                    'condDict': {'StorageElement': ['CERN-ARCHIVE', 'CERN-DST']}
                                    })
    self.assertEqual(res['OK'], True)
    self.assertEqual(res['Value'], {'graphDataDict': {'CERN-ARCHIVE': {1355616000: 2.34455676781291,
                                                                       1355702400: 2.5445567678129102},
                                                      'CERN-DST': {1355616000: 0.34455676781290995,
                                                                   1355702400: 0.54455676781290996}
                                                      },
                                    'data': {'CERN-ARCHIVE': {1355616000: 2344556.76781291,
                                                              1355702400: 2544556.76781291},
                                             'CERN-DST': {1355616000: 344556.76781290997,
                                                          1355702400: 544556.76781291002}
                                             },
                                    'unit': 'TB',
                                    'granularity': 86400
                                    })

  def test_reportPhysicalFiles(self):
    """test the method "_reportPhysicalFiles"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

#     res = obj._reportPhysicalFiles( { 'groupingFields' : [ 0, [ 'mehh' ], 'blah' ],
#                                       'startTime'      : 'startTime',
#                                       'endTime'        : 'endTime',
#                                       'condDict'       : {}
#                                      } )
#     self.assertEqual( res[ 'OK' ], False )
#     self.assertEqual( res[ 'Message' ], 'No connection' )

    # Changed mocked to run over different lines of code
    mockAccountingDB._getConnection.return_value = {'OK': True, 'Value': []}
    res = obj._reportPhysicalFiles({'grouping': 'NextToABeer',
                                    'groupingFields': [0, ['mehh'], 'blah'],
                                    'startTime': 'startTime',
                                    'endTime': 'endTime',
                                    'condDict': {}
                                    })
    self.assertEqual(res['OK'], True, msg='Expected S_OK')
    self.assertEqual(res['Value'], {'graphDataDict': {},
                                    'data': {},
                                    'unit': 'files',
                                    'granularity': 'BucketLength'
                                    })

    mockedData = (('CERN-ARCHIVE', 1355616000, 86400, Decimal('412658.91')),
                  ('CERN-ARCHIVE', 1355702400, 86400, Decimal('413658.91')),
                  ('CERN-BUFFER', 1355616000, 86400, Decimal('250658.91')),
                  ('CERN-BUFFER', 1355702400, 86400, Decimal('261658.91')),
                  ('CERN-DST', 1355616000, 86400, Decimal('186658.91')),
                  ('CERN-DST', 1355702400, 86400, Decimal('187658.91')))
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': mockedData}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 86400

    res = obj._reportPhysicalFiles({'grouping': 'StorageElement',
                                    'groupingFields': ('%s', ['StorageElement']),
                                    'startTime': 1355663249.0,
                                    'endTime': 1355749690.0,
                                    'condDict': {'StorageElement': ['CERN-ARCHIVE', 'CERN-DST', 'CERN-BUFFER']}
                                    })
    self.assertEqual(res['OK'], True)
    self.assertEqual(res['Value'], {'graphDataDict': {'CERN-BUFFER': {1355616000: 250.65890999999999,
                                                                      1355702400: 261.65890999999999},
                                                      'CERN-ARCHIVE': {1355616000: 412.65890999999999,
                                                                       1355702400: 413.65890999999999},
                                                      'CERN-DST': {1355616000: 186.65890999999999,
                                                                   1355702400: 187.65890999999999}
                                                      },
                                    'data': {'CERN-BUFFER': {1355616000: 250658.91,
                                                             1355702400: 261658.91},
                                             'CERN-ARCHIVE': {1355616000: 412658.90999999997,
                                                              1355702400: 413658.90999999997},
                                             'CERN-DST': {1355616000: 186658.91,
                                                          1355702400: 187658.91}
                                             },
                                    'unit': 'kfiles',
                                    'granularity': 86400
                                    })

  def test_multiplicityReport(self):
    """test the method "_multiplicityReport"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

#     res = obj._multiplicityReport( { 'groupingFields' : [ 0, [ 'mehh' ], 'blah' ],
#                                      'startTime'      : 'startTime',
#                                      'endTime'        : 'endTime',
#                                      'condDict'       : {}
#                                      }, '', '' )
#     self.assertEqual( res[ 'OK' ], False )
#     self.assertEqual( res[ 'Message' ], 'No connection' )
#
    # We are using mock 0.7, we need this little hack
    side_values = [{'OK': False, 'Message': 'No connection'}, {'OK': True, 'Value': []}]

    def side_effect():
      return side_values.pop()

    # Changed mocked to run over different lines of code
#     mockAccountingDB._getConnection.side_effect = side_effect
#     res = obj._multiplicityReport( { 'groupingFields' : [ 0, [ 'mehh' ], 'blah' ],
#                                      'startTime'      : 'startTime',
#                                      'endTime'        : 'endTime',
#                                      'condDict'       : {}
#                                     }, '', '' )
#     self.assertEqual( res[ 'OK' ], False )
#     self.assertEqual( res[ 'Message' ], 'No connection' )

    # Changed mocked to run over different lines of code
    mockAccountingDB._getConnection.return_value = {'OK': True, 'Value': []}

# ...............................................................................


class StoragePlotterUnitTestCrashes(StoragePlotterTestCase):
  """StoragePlotterUnitTestCrashes.

  <constructor>
   - test_instantiate
  <class variables>
  <methods>
   - test_reportCatalogSpace
   - test_plotCatalogSpace
   - test_reportCatalogFiles
   - test_plotCatalogFiles
   - test_reportPhysicalSpace
   - test_plotPhysicalSpace
   - test_reportPhysicalFiles
   - test_plotPhysicalFiles
  """

  def test_instantiate(self):
    """test the constructor."""

    self.assertRaises(TypeError, self.classsTested)
    self.assertRaises(TypeError, self.classsTested, None)
    self.assertRaises(TypeError, self.classsTested, None, None, None, None)

    self.assertRaises(TypeError, self.classsTested, extraArgs=None)
    self.assertRaises(TypeError, self.classsTested, None, extraArgs=None)
    self.assertRaises(TypeError, self.classsTested, None, None, None, extraArgs=None)

  def test_reportCatalogSpace(self):
    """test the method "_reportCatalogSpace"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

    self.assertRaises(KeyError, obj._reportCatalogSpace, {})
    self.assertRaises(KeyError, obj._reportCatalogSpace, {'grouping': 1})
    self.assertRaises(IndexError, obj._reportCatalogSpace, {'grouping': 1,
                                                            'groupingFields': []})
    self.assertRaises(TypeError, obj._reportCatalogSpace, {'grouping': 1,
                                                           'groupingFields': [1, 2]})
    self.assertRaises(TypeError, obj._reportCatalogSpace, {'grouping': 1,
                                                           'groupingFields': [1, [2]]})
    self.assertRaises(TypeError, obj._reportCatalogSpace, {'grouping': 1,
                                                           'groupingFields': ['1', '2']})
    self.assertRaises(KeyError, obj._reportCatalogSpace, {'grouping': 1,
                                                          'groupingFields': ['1', [2]]})
    self.assertRaises(KeyError, obj._reportCatalogSpace, {'grouping': 1,
                                                          'groupingFields': ['1', [2, 2]],
                                                          'startTime': None})
    self.assertRaises(KeyError, obj._reportCatalogSpace, {'grouping': 1,
                                                          'groupingFields': ['1', [2, 2]],
                                                          'startTime': None,
                                                          'endTime': None})
    self.assertRaises(TypeError, obj._reportCatalogSpace, {'grouping': 1,
                                                           'groupingFields': ['1', [2, 2]],
                                                           'startTime': None,
                                                           'endTime': None,
                                                           'condDict': None})

  def test_plotCatalogSpace(self):
    """test the method "_plotCatalogSpace"."""

    obj = self.classsTested(None, None)
    self.assertRaises(TypeError, obj._plotCatalogSpace, None, None, None)
    self.assertRaises(KeyError, obj._plotCatalogSpace, {}, None, None)
    self.assertRaises(KeyError, obj._plotCatalogSpace, {'startTime': 'startTime'},
                      None, None)
    self.assertRaises(TypeError, obj._plotCatalogSpace, {'startTime': 'startTime',
                                                         'endTime': 'endTime'},
                      None, None)
    self.assertRaises(KeyError, obj._plotCatalogSpace, {'startTime': 'startTime',
                                                        'endTime': 'endTime'},
                      {}, None)
    self.assertRaises(KeyError, obj._plotCatalogSpace, {'startTime': 'startTime',
                                                        'endTime': 'endTime'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotCatalogSpace, {'startTime': 'startTime',
                                                        'endTime': 'endTime',
                                                        'grouping': 'grouping'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotCatalogSpace, {'startTime': 'startTime',
                                                        'endTime': 'endTime',
                                                        'grouping': 'grouping'},
                      {'granularity': 'granularity',
                       'graphDataDict': 'graphDataDict'}, None)

  def test_reportCatalogFiles(self):
    """test the method "_reportCatalogFiles"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

    self.assertRaises(KeyError, obj._reportCatalogFiles, {})
    self.assertRaises(KeyError, obj._reportCatalogFiles, {'grouping': 1})
    self.assertRaises(IndexError, obj._reportCatalogFiles, {'grouping': 1,
                                                            'groupingFields': []})
    self.assertRaises(TypeError, obj._reportCatalogFiles, {'grouping': 1,
                                                           'groupingFields': [1, 2]})
    self.assertRaises(TypeError, obj._reportCatalogFiles, {'grouping': 1,
                                                           'groupingFields': [1, [2]]})
    self.assertRaises(TypeError, obj._reportCatalogFiles, {'grouping': 1,
                                                           'groupingFields': ['1', '2']})
    self.assertRaises(KeyError, obj._reportCatalogFiles, {'grouping': 1,
                                                          'groupingFields': ['1', [2]]})
    self.assertRaises(KeyError, obj._reportCatalogFiles, {'grouping': 1,
                                                          'groupingFields': ['1', [2, 2]],
                                                          'startTime': None})
    self.assertRaises(KeyError, obj._reportCatalogFiles, {'grouping': 1,
                                                          'groupingFields': ['1', [2, 2]],
                                                          'startTime': None,
                                                          'endTime': None})
    self.assertRaises(TypeError, obj._reportCatalogFiles, {'grouping': 1,
                                                           'groupingFields': ['1', [2, 2]],
                                                           'startTime': None,
                                                           'endTime': None,
                                                           'condDict': None})

  def test_plotCatalogFiles(self):
    """test the method "_plotCatalogFiles"."""

    obj = self.classsTested(None, None)
    self.assertRaises(TypeError, obj._plotCatalogFiles, None, None, None)
    self.assertRaises(KeyError, obj._plotCatalogFiles, {}, None, None)
    self.assertRaises(KeyError, obj._plotCatalogFiles, {'startTime': 'startTime'},
                      None, None)
    self.assertRaises(TypeError, obj._plotCatalogFiles, {'startTime': 'startTime',
                                                         'endTime': 'endTime'},
                      None, None)
    self.assertRaises(KeyError, obj._plotCatalogFiles, {'startTime': 'startTime',
                                                        'endTime': 'endTime'},
                      {}, None)
    self.assertRaises(KeyError, obj._plotCatalogFiles, {'startTime': 'startTime',
                                                        'endTime': 'endTime'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotCatalogFiles, {'startTime': 'startTime',
                                                        'endTime': 'endTime',
                                                        'grouping': 'grouping'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotCatalogFiles, {'startTime': 'startTime',
                                                        'endTime': 'endTime',
                                                        'grouping': 'grouping'},
                      {'granularity': 'granularity',
                       'graphDataDict': 'graphDataDict'}, None)

  def test_reportPhysicalSpace(self):
    """test the method "_reportPhysicalSpace"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

    self.assertRaises(KeyError, obj._reportPhysicalSpace, {})
    self.assertRaises(IndexError, obj._reportPhysicalSpace, {'groupingFields': []})
    self.assertRaises(TypeError, obj._reportPhysicalSpace, {'groupingFields': [1, 2]})
    self.assertRaises(TypeError, obj._reportPhysicalSpace, {'groupingFields': [1, [2]]})
    self.assertRaises(TypeError, obj._reportPhysicalSpace, {'groupingFields': ['1', '2']})
    self.assertRaises(KeyError, obj._reportPhysicalSpace, {'groupingFields': ['1', [2]]})
    self.assertRaises(KeyError, obj._reportPhysicalSpace, {'groupingFields': ['1', [2, 2]],
                                                           'startTime': None})
    self.assertRaises(KeyError, obj._reportPhysicalSpace, {'groupingFields': ['1', [2, 2]],
                                                           'startTime': None,
                                                           'endTime': None})
    self.assertRaises(TypeError, obj._reportPhysicalSpace, {'groupingFields': ['1', [2, 2]],
                                                            'startTime': None,
                                                            'endTime': None,
                                                            'condDict': None})

  def test_plotPhysicalSpace(self):
    """test the method "_plotPhysicalSpace"."""

    obj = self.classsTested(None, None)
    self.assertRaises(TypeError, obj._plotPhysicalSpace, None, None, None)
    self.assertRaises(KeyError, obj._plotPhysicalSpace, {}, None, None)
    self.assertRaises(KeyError, obj._plotPhysicalSpace, {'startTime': 'startTime'},
                      None, None)
    self.assertRaises(TypeError, obj._plotPhysicalSpace, {'startTime': 'startTime',
                                                          'endTime': 'endTime'},
                      None, None)
    self.assertRaises(KeyError, obj._plotPhysicalSpace, {'startTime': 'startTime',
                                                         'endTime': 'endTime'},
                      {}, None)
    self.assertRaises(KeyError, obj._plotPhysicalSpace, {'startTime': 'startTime',
                                                         'endTime': 'endTime'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotPhysicalSpace, {'startTime': 'startTime',
                                                         'endTime': 'endTime',
                                                         'grouping': 'grouping'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotPhysicalSpace, {'startTime': 'startTime',
                                                         'endTime': 'endTime',
                                                         'grouping': 'grouping'},
                      {'granularity': 'granularity',
                       'graphDataDict': 'graphDataDict'}, None)

  def test_reportPhysicalFiles(self):
    """test the method "_reportPhysicalFiles"."""

    mockAccountingDB = mock.Mock()
    mockAccountingDB._getConnection.return_value = {'OK': False, 'Message': 'No connection'}
    mockAccountingDB.retrieveBucketedData.return_value = {'OK': True, 'Value': []}
    mockAccountingDB.calculateBucketLengthForTime.return_value = 'BucketLength'
    obj = self.classsTested(mockAccountingDB, None)

    self.assertRaises(KeyError, obj._reportPhysicalFiles, {})
    self.assertRaises(IndexError, obj._reportPhysicalFiles, {'groupingFields': []})
    self.assertRaises(TypeError, obj._reportPhysicalFiles, {'groupingFields': [1, 2]})
    self.assertRaises(TypeError, obj._reportPhysicalFiles, {'groupingFields': [1, [2]]})
    self.assertRaises(TypeError, obj._reportPhysicalFiles, {'groupingFields': ['1', '2']})
    self.assertRaises(KeyError, obj._reportPhysicalFiles, {'groupingFields': ['1', [2]]})
    self.assertRaises(KeyError, obj._reportPhysicalFiles, {'groupingFields': ['1', [2, 2]],
                                                           'startTime': None})
    self.assertRaises(KeyError, obj._reportPhysicalFiles, {'groupingFields': ['1', [2, 2]],
                                                           'startTime': None,
                                                           'endTime': None})
    self.assertRaises(TypeError, obj._reportPhysicalFiles, {'groupingFields': ['1', [2, 2]],
                                                            'startTime': None,
                                                            'endTime': None,
                                                            'condDict': None})

  def test_plotPhysicalFiles(self):
    """test the method "_plotPhysicalFiles"."""

    obj = self.classsTested(None, None)
    self.assertRaises(TypeError, obj._plotPhysicalFiles, None, None, None)
    self.assertRaises(KeyError, obj._plotPhysicalFiles, {}, None, None)
    self.assertRaises(KeyError, obj._plotPhysicalFiles, {'startTime': 'startTime'},
                      None, None)
    self.assertRaises(TypeError, obj._plotPhysicalFiles, {'startTime': 'startTime',
                                                          'endTime': 'endTime'},
                      None, None)
    self.assertRaises(KeyError, obj._plotPhysicalFiles, {'startTime': 'startTime',
                                                         'endTime': 'endTime'},
                      {}, None)
    self.assertRaises(KeyError, obj._plotPhysicalFiles, {'startTime': 'startTime',
                                                         'endTime': 'endTime'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotPhysicalFiles, {'startTime': 'startTime',
                                                         'endTime': 'endTime',
                                                         'grouping': 'grouping'},
                      {'granularity': 'granularity'}, None)
    self.assertRaises(KeyError, obj._plotPhysicalFiles, {'startTime': 'startTime',
                                                         'endTime': 'endTime',
                                                         'grouping': 'grouping'},
                      {'granularity': 'granularity',
                       'graphDataDict': 'graphDataDict'}, None)

################################################################################


if __name__ == '__main__':
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(StoragePlotterTestCase)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(StoragePlotterUnitTest))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(StoragePlotterUnitTestCrashes))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""stores the replica readed from an xml."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import gLogger

__RCSID__ = "$Id$"


class Replica:
  """Replica class."""

  #############################################################################
  def __init__(self):
    """initialize the class members."""
    self.params_ = []
    self.fileName_ = ""

  #############################################################################
  def addParam(self, param):
    """sets the parameters."""
    self.params_ += [param]

  #############################################################################
  def getaprams(self):
    """returns the list of parameters."""
    return self.params_

  #############################################################################
  def getFileName(self):
    """returns the file name."""
    return self.fileName_

  #############################################################################
  def setFileName(self, name):
    """sets the file name."""
    self.fileName_ = name

  #############################################################################
  def __repr__(self):
    """It idents the print output."""
    result = "\nReplica: "
    result += self.fileName_ + "\n"
    for param in self.params_:
      result += str(param)

    return result

  #############################################################################
  def writeToXML(self):
    """writs an XML file."""
    gLogger.debug("Replica XML writing!!!")
    result = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Replicas SYSTEM "book.dtd">
<Replicas>
"""
    for param in self.getaprams():
      result += param.writeToXML(False)

    result += '</Replicas>'
    return result
  #############################################################################



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Get Data Quality Flag for the given run."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import DIRAC

from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base import Script

  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] ... Run ...' % Script.scriptName,
      'Arguments:',
      '  Run:      Run number']))
  Script.parseCommandLine(ignoreErrors=True)
  runSet = set(int(id) for arg in Script.getPositionalArgs() for id in arg.split(','))

  if not runSet:
    Script.showHelp(exitCode=1)

  gLogger.showHeaders(False)
  from LHCbDIRAC.BookkeepingSystem.Client.BookkeepingClient import BookkeepingClient
  cl = BookkeepingClient()

  gLogger.notice("Run Number".ljust(15) + "Stream".ljust(10) + "Flag".ljust(10))

  error = False
  for runId in sorted(runSet):
    retVal = cl.getRunFilesDataQuality(runId)
    if retVal['OK']:
      for run, stream, flag in sorted((run, stream, flag) for run, flag, stream in retVal["Value"]):
        gLogger.notice(str(run).ljust(15) + str(stream).ljust(10) + str(flag).ljust(10))
      gLogger.notice("-----------------------------------")
    else:
      gLogger.error(retVal["Message"])
      error = True

  if error:
    DIRAC.exit(1)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""LHCbDIRAC Job DB.

Extends the DIRAC JobDB with minor things
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC import S_OK
from DIRAC.Core.Utilities import Time

from DIRAC.WorkloadManagementSystem.DB.JobDB import JobDB as DIRACJobDB


class JobDB(DIRACJobDB):
  """Extension of the DIRAC Job DB."""

  def __init__(self):
    """The standard constructor takes the database name (dbname) and the name
    of the configuration section (dbconfig)"""
    DIRACJobDB.__init__(self)
    self.jdl2DBParameters += ['runNumber']

  def getTimings(self, site, period=3600):
    """Get CPU and wall clock times for the jobs finished in the last hour."""
    ret = self._escapeString(site)
    if not ret['OK']:
      return ret
    site = ret['Value']

    date = str(Time.dateTime() - Time.second * period)
    req = "SELECT JobID from Jobs WHERE Site=%s and EndExecTime > '%s' " % (site, date)
    result = self._query(req)
    jobList = [str(x[0]) for x in result['Value']]
    jobString = ','.join(jobList)

    req = "SELECT SUM(Value) from JobParameters WHERE Name='TotalCPUTime(s)' and JobID in (%s)" % jobString
    result = self._query(req)
    if not result['OK']:
      return result
    cpu = result['Value'][0][0]
    if not cpu:
      cpu = 0.0

    req = "SELECT SUM(Value) from JobParameters WHERE Name='WallClockTime(s)' and JobID in (%s)" % jobString
    result = self._query(req)
    if not result['OK']:
      return result
    wctime = result['Value'][0][0]
    if not wctime:
      wctime = 0.0

    return S_OK({"CPUTime": int(cpu), "WallClockTime": int(wctime)})
